/*
 * term.h
 */

extern int cols, lines;		/* terminal window size */
extern int cols_1, lines_1;	/* edit line width, last line */
extern int line0, col0;		/* origin of input line */

#if defined(VT100) || defined(HC_ISO_LATIN1)
#define metakey 0
#else
extern int metakey;
#endif

#ifndef VT100

typedef struct {
    int cost;
    char str[CAPLEN];
} tc_string;
extern tc_string tcap_kl, tcap_kr, tcap_ku, tcap_kd;
#define key_left	tcap_kl.str
#define key_right	tcap_kr.str
#define key_up		tcap_ku.str
#define key_down	tcap_kd.str

#else

typedef const char tc_string[];
extern tc_string key_left, key_right, key_up, key_down;

#endif

extern char *terminal_type;

extern int edattr;		/* edit line attributes code */
extern char edattrbeg[], edattrend[];	/* and on/off sequences */

void init_termcap();
void keypad_bindings();
void default_bindings();
void moveto();
void lastline();
void check_line0();
void newline();
void display_footnote();
void run_command();
void skip_edline();
void clear_input();
void redraw_input();
void clear_prompt();
void redraw_prompt();
void clear_all();
void redraw_all();
void delete_chars();
void insert_chars();
void insert_one_char();
int replace_char();
void attr_string();
int set_stdcolour();
char *wrap_out();
void to_pager();
void set_terminal();
void reset_terminal();
void set_spec_keys();
int check_winsize();

/* macros to find cursor position from input buffer position */
/* NOTE: use unsigned division perhaps? */
#define CURLINE(pos) (((pos) + col0) / cols_1 + line0)
#define CURCOL(pos) (((pos) + col0) % cols_1)

/* NOTE: anyone knows if ANSI 6429 talks about more than 8 colours? */
typedef enum {
    ATTR_BLACK, ATTR_RED, ATTR_GREEN, ATTR_YELLOW,
    ATTR_BLUE, ATTR_MAGENTA, ATTR_CYAN, ATTR_WHITE,
    ATTR_FILL, ATTR_COLOURS = ATTR_FILL, ATTR_NONE,
    ATTR_BOLD, ATTR_UNDERLINE, ATTR_INVERSE, ATTR_REVERSE,
    ATTR_ERROR
} attr_token;

#define ATTR_NAMES {				\
    "black", "red", "green", "yellow",		\
    "blue", "magenta", "cyan", "white",		\
    "on", "none",				\
    "bold", "underline", "inverse", "reverse",	\
    (char *)0					\
}

/* constructors / accessors for attribute codes */
#define ATTRCODE(fg, bg) ((fg) + ((bg) << 8))
#define FOREGROUND(attrcode) ((attrcode) & 0xff)
#define BACKGROUND(attrcode) ((attrcode) >> 8)

#define NOATTRCODE ATTRCODE(ATTR_NONE, ATTR_NONE)

#ifdef TELNETBUG
/*
 * NCSA telnet 2.2 doesn't reset the colour when it receives "esc [ m",
 * so we must know what the normal colours are in order to reset it.
 * These colours can be changed with the #stdcolour command.
 */
#define DEFAULT_ATTRCODE ATTRCODE(ATTR_WHITE, ATTR_BLUE)
#endif

