/*
 * util.c
 * source file for cancan
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

FILE *popen();
int pclose();

#include "cancan.h"
#include "list.h"

static char **make_env();

int do_fork()
{
    int pid;
    flush_output_buffer();
    fflush(stdout);
    pid = fork();
    if(pid < 0)
	printf("#fork failed, sorry\n");
    return pid;
}

/*
 * execute a command with current environment
 */
void do_execve(args)
char **args;
{
    signal(SIGINT, SIG_DFL);	/* the only sig that's not DFL yet */
    execve(*args, args, make_env());
    _exit(1);
}

/*
 * this shuffling with env in parent process is darned ugly,
 * but i doubt if there's a better way.
 */
FILE *do_popen(cmd, mode)
char *cmd, *mode;
{
    char **temp_env = make_env();
    char **back_env = environ;
    FILE *fp;
    environ = temp_env;
    fflush(stdout);
    fp = popen(cmd, mode);
    environ = back_env;
    free((char*)temp_env);
    return fp;
}

int do_pclose(fp)
FILE *fp;
{
    return pclose(fp);
}

static char **make_env()
{
    char **new;
    varnode *p, *vars = make_varlist();
    int n;
    for(n = 0, p = vars; p; p = p->next)
	n++;
    new = (char **)malloc((n + 1) * sizeof(char *));
    if(!new) syserr("malloc");
    for(n = 0, p = vars; p; p = p->next)
	new[n++] = p->name;
    new[n] = 0;
    return new;
}


lbuf *lbuf_alloc(size)
int size;
{
    lbuf *b = (lbuf *)malloc(size + sizeof(*b) - sizeof((*b).buf));
    if(!b) syserr("malloc");
    b->pos = 0; b->size = size;
    return b;
}

void lbuf_free(bp)
lbuf **bp;
{
    lbuf *b = *bp;
    if(b) {
	*bp = 0;
	free((char *)b);
    }
}

int lbuf_accum(b, c)
lbuf *b; int c;
{
    return LBUF_ACCUM(b, c);
}


int str_len(str)
char *str;
{
    return strlen(str);
}

/*
 * copy a zero-terminated string and return it's length
 */
int str_copy(dst, src)
char *dst, *src;
{
    register char *s = src, *d = dst, c;
    do {
	c = *s++;
    } while((*d++ = c));
    return d - dst - 1;
}

char *str_dup(s)
char *s;
{
    return strn_dup(s, (int)strlen(s));
}

char *strn_dup(s, n)
char *s; int n;
{
    char *k = (char *)malloc(n + 1);
    if(!k) syserr("malloc");
    memcpy(k, s, n);  k[n] = 0;
    return k;
}

void str_assign(str, s)
char **str, *s;
{
    char *new = 0;
    if(s) new = str_dup(s);
    if(*str) free(*str);
    *str = new;
}

/*
 * TODO: special variables (varnode->var_func):
 * error, RANDOM, FOOTNOTE, PROMPT, PREFIX,
 * escape-char, SAVEFILE, ...
 */

/*
 * analogous to getenv, but with hash lookup
 */
char *get_env(var)
char *var;
{
    return get_envn(var, str_len(var));
}

/*
 * look up a variable
 */
char *get_envn(var, len)
char *var; int len;
{
    varnode *p = *lookup_varhash(var, len);
    return !p ? 0 : p->name + p->nlen + 1;
}

/*
 * assign to a variable
 */
void put_env(var, value)
char *var;
char *value;
{
    char buf[LINELEN];
    sprintf(buf, "%s=%s", var, value);
    put_envn(buf, str_len(var), 1);
}

/*
 * set (or unset if mode == 0) a variable
 * var should contain "name=value", and len the length of name
 */
void put_envn(var, len, mode)
char *var; int len;
int mode;
{
    varnode **pp = lookup_varhash(var, len);
    if(mode) {
	if(*pp) {
	    str_assign(&(*pp)->name, var);
	} else {
	    varnode *p = new_varnode(var, len);
	    hash_insert_(varhash, p);
	}
    } else if(*pp)
	free_node_(hash_remove_(pp));
}

/*
 * initialize our variables
 */
void init_env()
{
    char *p, **pp = environ;
    while((p = *pp++)) {
	char *s = strchr(p, '=');
	if(s) { put_envn(p, s - p, 1); }
    }
}

